package com.backendless.backendless_sdk.call_handlers;

import android.util.Log;

import com.backendless.Backendless;
import com.backendless.async.callback.AsyncCallback;
import com.backendless.async.callback.BackendlessCallback;
import com.backendless.exceptions.BackendlessFault;
import com.backendless.backendless_sdk.utils.FlutterCallback;
import com.backendless.files.FileInfo;
import com.backendless.geo.GeoCluster;
import com.backendless.geo.GeoPoint;
import com.backendless.geo.SearchMatchesResult;
import com.backendless.messaging.MessageStatus;
import com.backendless.messaging.PublishStatusEnum;
import com.backendless.persistence.DataQueryBuilder;
import com.backendless.persistence.LoadRelationsQueryBuilder;
import com.backendless.property.ObjectProperty;

import java.util.Date;
import java.util.List;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

public class CacheCallHandler implements MethodChannel.MethodCallHandler {

    public CacheCallHandler() {
        
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        switch (call.method) {
            case "Backendless.Cache.contains":
                contains(call, result);
                break;
            case "Backendless.Cache.delete":
                delete(call, result);
                break;
            case "Backendless.Cache.expireAt":
                expireAt(call, result);
                break;
            case "Backendless.Cache.expireIn":
                expireIn(call, result);
                break;
            case "Backendless.Cache.get":
                get(call, result);
                break;
            case "Backendless.Cache.put":
                put(call, result);
                break;
            default:
                result.notImplemented();
        }
    }

    private void contains(MethodCall call, MethodChannel.Result result) {
        String key = call.argument("key");
        Backendless.Cache.contains(key, new FlutterCallback<Boolean>(result));
    }

    private void delete(MethodCall call, MethodChannel.Result result) {
        String key = call.argument("key");
        Backendless.Cache.delete(key, new FlutterCallback<>(result));
    }

    private void expireAt(MethodCall call, MethodChannel.Result result) {
        String key = call.argument("key");
        Date date = call.argument("date");
        Long timestamp = call.argument("timestamp");
        FlutterCallback<Object> callback = new FlutterCallback<>(result);
        if (date != null) {
            Backendless.Cache.expireAt(key, date, callback);
        } else if (timestamp != null) {
            Backendless.Cache.expireAt(key, timestamp, callback);
        }
    }

    private void expireIn(MethodCall call, MethodChannel.Result result) {
        String key = call.argument("key");
        Integer seconds = call.argument("seconds");
        Backendless.Cache.expireIn(key, seconds, new FlutterCallback<>(result));
    }

    private void get(MethodCall call, final MethodChannel.Result result) {
        String key = call.argument("key");
//        Pass AsyncCallback for compatibility
        Backendless.Cache.get(key, new AsyncCallback<Object>() {
            @Override
            public void handleResponse(Object response) {
                result.success(response);
            }

            @Override
            public void handleFault(BackendlessFault fault) {
                result.error(fault.getCode(), fault.getMessage(), fault.getDetail());
            }
        });
    }

    private void put(MethodCall call, MethodChannel.Result result) {
        String key = call.argument("key");
        Object object = call.argument("object");
        Integer timeToLive = call.argument("timeToLive");

        FlutterCallback<Object> callback = new FlutterCallback<>(result);
        if (timeToLive != null) {
            Backendless.Cache.put(key, object, timeToLive, callback);
        } else {
            Backendless.Cache.put(key, object, callback);
        }
    }
}
